@@ -246,121 +246,129 @@ Remove the undo log segment from the rseg slot if it is too big for reuse.
 void
 trx_purge_add_undo_to_history(const trx_t* trx, trx_undo_t*& undo, mtr_t* mtr)
 {
-  DBUG_PRINT("trx", ("commit(" TRX_ID_FMT "," TRX_ID_FMT ")",
-                     trx->id, trx_id_t{trx->rw_trx_hash_element->no}));
-  ut_ad(undo->id < TRX_RSEG_N_SLOTS);
-  ut_ad(undo == trx->rsegs.m_redo.undo);
-  trx_rseg_t *rseg= trx->rsegs.m_redo.rseg;
-  ut_ad(undo->rseg == rseg);
-  buf_block_t *rseg_header= rseg->get(mtr, nullptr);
-  /* We are in transaction commit; we cannot return an error. If the
-  database is corrupted, it is better to crash it than to
-  intentionally violate ACID by committing something that is known to
-  be corrupted. */
-  ut_ad(rseg_header);
-  buf_block_t *undo_page=
-    buf_page_get(page_id_t(rseg->space->id, undo->hdr_page_no), 0,
-                 RW_X_LATCH, mtr);
-  /* This function is invoked during transaction commit, which is not
-  allowed to fail. If we get a corrupted undo header, we will crash here. */
-  ut_a(undo_page);
-  trx_ulogf_t *undo_header= undo_page->page.frame + undo->hdr_offset;
+	DBUG_PRINT("trx", ("commit(" TRX_ID_FMT "," TRX_ID_FMT ")",
+			   trx->id, trx_id_t{trx->rw_trx_hash_element->no}));
+	ut_ad(undo == trx->rsegs.m_redo.undo);
+	trx_rseg_t*	rseg		= trx->rsegs.m_redo.rseg;
+	ut_ad(undo->rseg == rseg);
+	buf_block_t*	rseg_header	= rseg->get(mtr, nullptr);
+	/* We are in transaction commit; we cannot return an error. If the
+	database is corrupted, it is better to crash it than to
+	intentionally violate ACID by committing something that is known to
+	be corrupted. */
+	ut_ad(rseg_header);
+	buf_block_t*	undo_page	= trx_undo_set_state_at_finish(
+		undo, mtr);
+	trx_ulogf_t*	undo_header	= undo_page->page.frame
+		+ undo->hdr_offset;
 
-  ut_ad(mach_read_from_2(undo_header + TRX_UNDO_NEEDS_PURGE) <= 1);
-  ut_ad(rseg->needs_purge > trx->id);
+	ut_ad(mach_read_from_2(undo_header + TRX_UNDO_NEEDS_PURGE) <= 1);
+	ut_ad(rseg->needs_purge > trx->id);
 
-  if (rseg->last_page_no == FIL_NULL)
-  {
-    rseg->last_page_no= undo->hdr_page_no;
-    rseg->set_last_commit(undo->hdr_offset, trx->rw_trx_hash_element->no);
-  }
+	if (UNIV_UNLIKELY(mach_read_from_4(TRX_RSEG + TRX_RSEG_FORMAT
+					   + rseg_header->page.frame))) {
+		/* This database must have been upgraded from
+		before MariaDB 10.3.5. */
+		trx_rseg_format_upgrade(rseg_header, mtr);
+	}
 
-  rseg->history_size++;
+	if (undo->state != TRX_UNDO_CACHED) {
+		/* The undo log segment will not be reused */
+		ut_a(undo->id < TRX_RSEG_N_SLOTS);
+		static_assert(FIL_NULL == 0xffffffff, "");
+		mtr->memset(rseg_header,
+			    TRX_RSEG + TRX_RSEG_UNDO_SLOTS
+			    + undo->id * TRX_RSEG_SLOT_SIZE, 4, 0xff);
 
-  if (UNIV_UNLIKELY(mach_read_from_4(TRX_RSEG + TRX_RSEG_FORMAT +
-                                     rseg_header->page.frame)))
-    /* This database must have been upgraded from before MariaDB 10.3.5. */
-    trx_rseg_format_upgrade(rseg_header, mtr);
+		uint32_t hist_size = mach_read_from_4(
+			TRX_RSEG_HISTORY_SIZE + TRX_RSEG
+			+ rseg_header->page.frame);
 
-  uint16_t undo_state;
+		ut_ad(undo->size == flst_get_len(TRX_UNDO_SEG_HDR
+						 + TRX_UNDO_PAGE_LIST
+						 + undo_page->page.frame));
 
-  if (undo->size == 1 &&
-      TRX_UNDO_PAGE_REUSE_LIMIT >
-      mach_read_from_2(TRX_UNDO_PAGE_HDR + TRX_UNDO_PAGE_FREE +
-                       undo_page->page.frame))
-  {
-    undo->state= undo_state= TRX_UNDO_CACHED;
-    UT_LIST_ADD_FIRST(rseg->undo_cached, undo);
-  }
-  else
-  {
-    ut_ad(undo->size == flst_get_len(TRX_UNDO_SEG_HDR + TRX_UNDO_PAGE_LIST +
-                                     undo_page->page.frame));
-    /* The undo log segment will not be reused */
-    static_assert(FIL_NULL == 0xffffffff, "");
-    mtr->memset(rseg_header, TRX_RSEG + TRX_RSEG_UNDO_SLOTS +
-                undo->id * TRX_RSEG_SLOT_SIZE, 4, 0xff);
-    uint32_t hist_size= mach_read_from_4(TRX_RSEG_HISTORY_SIZE + TRX_RSEG +
-                                         rseg_header->page.frame);
-    mtr->write<4>(*rseg_header, TRX_RSEG + TRX_RSEG_HISTORY_SIZE +
-                  rseg_header->page.frame, hist_size + undo->size);
-    mtr->write<8>(*rseg_header, TRX_RSEG + TRX_RSEG_MAX_TRX_ID +
-                  rseg_header->page.frame, trx_sys.get_max_trx_id());
-    ut_free(undo);
-    undo_state= TRX_UNDO_TO_PURGE;
-  }
+		mtr->write<4>(*rseg_header, TRX_RSEG + TRX_RSEG_HISTORY_SIZE
+			      + rseg_header->page.frame,
+			      hist_size + undo->size);
+		mtr->write<8>(*rseg_header, TRX_RSEG + TRX_RSEG_MAX_TRX_ID
+			      + rseg_header->page.frame,
+			      trx_sys.get_max_trx_id());
+	}
 
-  undo= nullptr;
+	/* After the purge thread has been given permission to exit,
+	we may roll back transactions (trx->undo_no==0)
+	in THD::cleanup() invoked from unlink_thd() in fast shutdown,
+	or in trx_rollback_recovered() in slow shutdown.
 
-  /* After the purge thread has been given permission to exit,
-  we may roll back transactions (trx->undo_no==0)
-  in THD::cleanup() invoked from unlink_thd() in fast shutdown,
-  or in trx_rollback_recovered() in slow shutdown.
+	Before any transaction-generating background threads or the
+	purge have been started, we can
+	start transactions in row_merge_drop_temp_indexes(),
+	and roll back recovered transactions.
 
-  Before any transaction-generating background threads or the purge
-  have been started, we can start transactions in
-  row_merge_drop_temp_indexes(), and roll back recovered transactions.
+	Arbitrary user transactions may be executed when all the undo log
+	related background processes (including purge) are disabled due to
+	innodb_force_recovery=2 or innodb_force_recovery=3.
+	DROP TABLE may be executed at any innodb_force_recovery	level.
 
-  Arbitrary user transactions may be executed when all the undo log
-  related background processes (including purge) are disabled due to
-  innodb_force_recovery=2 or innodb_force_recovery=3.  DROP TABLE may
-  be executed at any innodb_force_recovery level.
+	During fast shutdown, we may also continue to execute
+	user transactions. */
+	ut_ad(srv_undo_sources
+	      || trx->undo_no == 0
+	      || (!purge_sys.enabled()
+		  && (srv_is_being_started
+		      || trx_rollback_is_active
+		      || srv_force_recovery >= SRV_FORCE_NO_BACKGROUND))
+	      || srv_fast_shutdown);
 
-  During fast shutdown, we may also continue to execute user
-  transactions. */
-  ut_ad(srv_undo_sources || trx->undo_no == 0 ||
-        (!purge_sys.enabled() &&
-         (srv_is_being_started ||
-          trx_rollback_is_active ||
-          srv_force_recovery >= SRV_FORCE_NO_BACKGROUND)) ||
-        srv_fast_shutdown);
-
-#ifdef WITH_WSREP
-  if (wsrep_is_wsrep_xid(&trx->xid))
-    trx_rseg_update_wsrep_checkpoint(rseg_header, &trx->xid, mtr);
+#ifdef	WITH_WSREP
+	if (wsrep_is_wsrep_xid(&trx->xid)) {
+		trx_rseg_update_wsrep_checkpoint(rseg_header, &trx->xid, mtr);
+	}
 #endif
 
-  if (trx->mysql_log_file_name && *trx->mysql_log_file_name)
-    /* Update the latest binlog name and offset if log_bin=ON or this
-    is a replica. */
-    trx_rseg_update_binlog_offset(rseg_header, trx, mtr);
+	if (trx->mysql_log_file_name && *trx->mysql_log_file_name) {
+		/* Update the latest MySQL binlog name and offset info
+		in rollback segment header if MySQL binlogging is on
+		or the database server is a MySQL replication save. */
+		trx_rseg_update_binlog_offset(
+			rseg_header, trx->mysql_log_file_name,
+			trx->mysql_log_offset, mtr);
+	}
 
-  /* Add the log as the first in the history list */
+	/* Add the log as the first in the history list */
 
-  /* We are in transaction commit; we cannot return an error
-  when detecting corruption. It is better to crash the server
-  than to intentionally violate ACID by committing something
-  that is known to be corrupted. */
-  ut_a(flst_add_first(rseg_header, TRX_RSEG + TRX_RSEG_HISTORY, undo_page,
-                      uint16_t(page_offset(undo_header) +
-                               TRX_UNDO_HISTORY_NODE), mtr) == DB_SUCCESS);
+	/* We are in transaction commit; we cannot return an error
+	when detecting corruption. It is better to crash the server
+	than to intentionally violate ACID by committing something
+	that is known to be corrupted. */
+	ut_a(flst_add_first(rseg_header, TRX_RSEG + TRX_RSEG_HISTORY, undo_page,
+			    static_cast<uint16_t>(undo->hdr_offset
+						  + TRX_UNDO_HISTORY_NODE),
+			    mtr) == DB_SUCCESS);
 
-  mtr->write<2>(*undo_page, TRX_UNDO_SEG_HDR + TRX_UNDO_STATE +
-                undo_page->page.frame, undo_state);
-  mtr->write<8,mtr_t::MAYBE_NOP>(*undo_page, undo_header + TRX_UNDO_TRX_NO,
-                                 trx->rw_trx_hash_element->no);
-  mtr->write<2,mtr_t::MAYBE_NOP>(*undo_page, undo_header +
-                                 TRX_UNDO_NEEDS_PURGE, 1U);
+	mtr->write<8,mtr_t::MAYBE_NOP>(*undo_page,
+				       undo_header + TRX_UNDO_TRX_NO,
+				       trx->rw_trx_hash_element->no);
+	mtr->write<2,mtr_t::MAYBE_NOP>(*undo_page, undo_header
+				       + TRX_UNDO_NEEDS_PURGE, 1U);
+
+	if (rseg->last_page_no == FIL_NULL) {
+		rseg->last_page_no = undo->hdr_page_no;
+		rseg->set_last_commit(undo->hdr_offset,
+				      trx->rw_trx_hash_element->no);
+	}
+
+	rseg->history_size++;
+
+	if (undo->state == TRX_UNDO_CACHED) {
+		UT_LIST_ADD_FIRST(rseg->undo_cached, undo);
+	} else {
+		ut_ad(undo->state == TRX_UNDO_TO_PURGE);
+		ut_free(undo);
+	}
+
+	undo = NULL;
 }
 
 /** Free an undo log segment.
@@ -560,11 +568,10 @@ __attribute__((optimize(0)))
 # endif
 #endif
 /**
-Remove unnecessary history data from rollback segments. NOTE that when this
-function is called, the caller (purge_coordinator_callback)
-must not have any latches on undo log pages!
+Removes unnecessary history data from rollback segments. NOTE that when this
+function is called, the caller must not have any latches on undo log pages!
 */
-TRANSACTIONAL_TARGET void trx_purge_truncate_history()
+TRANSACTIONAL_TARGET static void trx_purge_truncate_history()
 {
   ut_ad(purge_sys.head <= purge_sys.tail);
   purge_sys_t::iterator &head= purge_sys.head.trx_no
@@ -586,7 +593,7 @@ TRANSACTIONAL_TARGET void trx_purge_truncate_history()
       if (dberr_t e=
           trx_purge_truncate_rseg_history(rseg, head,
                                           !rseg.is_referenced() &&
-                                          purge_sys.sees(rseg.needs_purge)))
+                                          rseg.needs_purge <= head.trx_no))
         err= e;
       rseg.latch.wr_unlock();
     }
@@ -600,11 +607,11 @@ TRANSACTIONAL_TARGET void trx_purge_truncate_history()
     {
       const ulint threshold=
         ulint(srv_max_undo_log_size >> srv_page_size_shift);
-      for (ulint i= purge_sys.truncate.last
+      for (uint32_t i= purge_sys.truncate.last
            ? purge_sys.truncate.last->id - srv_undo_space_id_start : 0,
            j= i;; )
       {
-        const auto space_id= srv_undo_space_id_start + i;
+        const uint32_t space_id= srv_undo_space_id_start + i;
         ut_ad(srv_is_undo_tablespace(space_id));
         fil_space_t *space= fil_space_get(space_id);
         ut_a(UT_LIST_GET_LEN(space->chain) == 1);
@@ -644,7 +651,7 @@ TRANSACTIONAL_TARGET void trx_purge_truncate_history()
 
       rseg.latch.rd_lock(SRW_LOCK_CALL);
       ut_ad(rseg.skip_allocation());
-      if (rseg.is_referenced() || !purge_sys.sees(rseg.needs_purge))
+      if (rseg.is_referenced() || rseg.needs_purge > head.trx_no)
       {
 not_free:
         rseg.latch.rd_unlock();
@@ -658,7 +665,7 @@ not_free:
       for (const trx_undo_t *undo= UT_LIST_GET_FIRST(rseg.undo_cached); undo;
            undo= UT_LIST_GET_NEXT(undo_list, undo))
       {
-        if (head.trx_no && head.trx_no < undo->trx_id)
+        if (head.trx_no < undo->trx_id)
           goto not_free;
         else
           cached+= undo->size;
@@ -791,7 +798,7 @@ not_free:
         continue;
 
       ut_ad(!rseg.is_referenced());
-      ut_ad(!head.trx_no || rseg.needs_purge <= head.trx_no);
+      ut_ad(rseg.needs_purge <= head.trx_no);
 
       buf_block_t *rblock= trx_rseg_header_create(&space,
                                                   &rseg - trx_sys.rseg_array,
@@ -1264,8 +1271,10 @@ TRANSACTIONAL_INLINE void purge_sys_t::clone_end_view()
 Run a purge batch.
 @param n_tasks       number of purge tasks to submit to the queue
 @param history_size  trx_sys.history_size()
+@param truncate      whether to truncate the history at the end of the batch
 @return number of undo log pages handled in the batch */
-TRANSACTIONAL_TARGET ulint trx_purge(ulint n_tasks, ulint history_size)
+TRANSACTIONAL_TARGET
+ulint trx_purge(ulint n_tasks, ulint history_size, bool truncate)
 {
 	que_thr_t*	thr = NULL;
 	ulint		n_pages_handled;
@@ -1317,6 +1326,10 @@ TRANSACTIONAL_TARGET ulint trx_purge(ulint n_tasks, ulint history_size)
 
 	purge_sys.clone_end_view();
 
+	if (truncate) {
+		trx_purge_truncate_history();
+	}
+
 	MONITOR_INC_VALUE(MONITOR_PURGE_INVOKED, 1);
 	MONITOR_INC_VALUE(MONITOR_PURGE_N_PAGE_HANDLED, n_pages_handled);
 
